package org.jboss.qe.cxf.ws.jms.jms;

import static org.hamcrest.CoreMatchers.containsString;

import org.apache.cxf.transport.jms.JMSConfigFeature;
import org.apache.cxf.transport.jms.JMSConfiguration;

import org.jboss.qe.cxf.ws.jms.GreetingService;

import org.junit.Assert;
import org.junit.Test;

import javax.xml.ws.WebServiceFeature;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import lombok.extern.slf4j.Slf4j;

/**
 * Reproducer for https://issues.jboss.org/browse/ENTESB-5907.
 */
@Slf4j
public class SoapJmsReplyToDestinationTest extends AbstractSoapJmsTest {

	protected static final String SERVICE_NAME = "JMSQueueGreetingService";
	protected static final String PORT_NAME = "JMSQueueGreetingPort";

	protected static final String SOAP_JMS_REQUEST_QUEUE = System.getProperty("soap.jms.request.queue");
	protected static final String SOAP_JMS_RESPONSE_QUEUE = System.getProperty("soap.jms.response.queue");

	/**
	 * Client listens to the same destination as was set in "replyTo" header.
	 */
	@Test
	public void listenAndDelegateToTheSameQueueTest() throws Exception {
		final WebServiceFeature[] features = new WebServiceFeature[] {
				createConnectionFactoryFeature(),
				createJMSConfigFeature(SOAP_JMS_REQUEST_QUEUE, SOAP_JMS_RESPONSE_QUEUE)
		};
		final GreetingService service = getWebServiceProxy(GreetingService.class, SERVICE_NAME, PORT_NAME, features);
		final String response = service.greet(REQUEST);
		Assert.assertEquals(String.format(RESPONSE_TEMPLATE, REQUEST, "queue"), response);
	}

	/**
	 * Client listens to a different destination than it was set in "replyTo" header.
	 */
	@Test
	public void notListenAndDelegateToTheSameQueueTest() throws Exception {
		final String differentQueue = "differentQueue";
		final JMSConfigFeature jcf = createJMSConfigFeature(SOAP_JMS_REQUEST_QUEUE, SOAP_JMS_RESPONSE_QUEUE);
		jcf.getJmsConfig().setReplyToDestination(differentQueue);

		final WebServiceFeature[] features = new WebServiceFeature[] {createConnectionFactoryFeature(), jcf};

		final ExecutorService executor = Executors.newFixedThreadPool(2);

		// Receive message from "differentQueue" asynchronously
		final Future<String> future = executor.submit(new Callable<String>() {

			public String call() throws Exception {
				return JmsClient.receiveFromQueue(getConnectionFactory(), differentQueue, TimeUnit.SECONDS.toMillis(8));
			}
		});

		// Call web service with timeout
		try {
			executor.submit(new Callable<String>() {
				public String call() throws Exception {
					Thread.sleep(500);
					final GreetingService service = getWebServiceProxy(GreetingService.class, SERVICE_NAME, PORT_NAME, features);
					return service.greet(REQUEST);
				}
			}).get(5, TimeUnit.SECONDS);
		} catch (TimeoutException e) {
			log.info(e.getMessage()); // expected exception because the response is in another queue
		}

		// Obtain SOAP message from "differentQueue"
		final String message = future.get(8, TimeUnit.SECONDS);

		Assert.assertThat(message, containsString(String.format(RESPONSE_TEMPLATE, REQUEST, "queue")));
	}

	protected JMSConfigFeature createJMSConfigFeature(String fromDest, String toDest) throws Exception {
		final JMSConfigFeature jcf = new JMSConfigFeature();
		final JMSConfiguration jmsConfig = new JMSConfiguration();
		jmsConfig.setConnectionFactory(getConnectionFactory());
		jmsConfig.setRequestURI(fromDest);
		jmsConfig.setTargetDestination(fromDest);
		// replyDestination = Destination name to listen on for reply messages
		jmsConfig.setReplyDestination(toDest);
		// replyToDestination = Destination name to send out as replyTo address in the message
		jmsConfig.setReplyToDestination(toDest);
		jcf.setJmsConfig(jmsConfig);
		return jcf;
	}
}
